function result = CytoSpectre(inputimage,varargin)

% CytoSpectre - spectral analysis of microscopy images
%
%   This function performs the CytoSpectre main analysis for a single image
%   and returns a results structure. Analysis settings can be provided as a
%   settings structure or omitted to use default settings. A logical mask
%   image can be optionally provided as the third input argument. The mask
%   is used for segmenting cells from the image and the analysis is then
%   performed separately for each cell.
%
%   result = CytoSpectre(inputimage)
%   result = CytoSpectre(inputimage,settings)
%   result = CytoSpectre(inputimage,settings,segmentationmask)
%   
%   inputimage          The image to analyze as a numerical matrix.
%                       Grayscale and RGB images are supported.
%   
%   settings            A structure of settings to use for the analysis.
%                       See the CytoSpectre user guide for details.
%
%   segmentationmask    A mask image matrix with logical true for pixels
%                       belonging to cells and logical false for pixels
%                       belonging to background. Size of the image should
%                       be the same as the size of inputimage.
%
%   result              A structure containing the results of the analysis.
%                       See the CytoSpectre user guide for details.
%

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% Check number of input arguments.
if nargin < 1 || nargin > 3
    error('Incorrect number of input arguments!');
end
% Check number of output arguments.
if nargout < 1 || nargout > 2
    error('Incorrect number of output arguments!');
end
% Check that the input image is at least two dimensional.
if length(size(inputimage)) < 2
    error('Input image is not an image matrix!');
end
% Check that the optional second input argument is a struct.
if nargin > 1
    if ~isstruct(varargin{1})
        error('Second input argument is not a struct!');
    end
end
% Check that the optional third input argument is two dimensional.
if nargin == 3
    if length(size(varargin{2})) ~= 2
        error('Segmentation mask input is not a 2D image matrix!');
    end
end

% Set settings according to user preference or set to default.
% Segmentation.
if nargin == 3
    analysissettings.issegmentationon = 1;
else
    analysissettings.issegmentationon = 0;
end
% Magnification, camera pixel size etc.
if nargin > 1 && isfield(varargin{1},'magnification')
    analysissettings.magnification = varargin{1}.magnification;
    if isempty(analysissettings.magnification) || ~isnumeric(analysissettings.magnification) || ~isfinite(analysissettings.magnification) || analysissettings.magnification <= 0
        error('Invalid magnification value!');
    end
else
    analysissettings.magnification = 40;
end
if nargin > 1 && isfield(varargin{1},'camerapixelsize')
    analysissettings.camerapixelsize = varargin{1}.camerapixelsize;
    if isempty(analysissettings.camerapixelsize) || ~isnumeric(analysissettings.camerapixelsize) || ~isfinite(analysissettings.camerapixelsize) || analysissettings.camerapixelsize <= 0
        error('Invalid camera pixel size value!');
    end
else
    analysissettings.camerapixelsize = 6.8;
end
analysissettings.imagepixelsize = analysissettings.camerapixelsize/analysissettings.magnification;
analysissettings.resolution = 1/analysissettings.imagepixelsize;
% Spectral resolution.
if nargin > 1 && isfield(varargin{1},'resolutionparameter')
    analysissettings.resolutionparameter = varargin{1}.resolutionparameter;
    if ~(analysissettings.resolutionparameter == 1 || analysissettings.resolutionparameter == 2 || analysissettings.resolutionparameter == 3 || analysissettings.resolutionparameter == 4 || analysissettings.resolutionparameter == 5)
        error('Invalid spectral resolution parameter value!');
    end
else
    analysissettings.resolutionparameter = 3;
end
% Color channel
if nargin > 1 && isfield(varargin{1},'targetchannel')
    analysissettings.targetchannel = varargin{1}.targetchannel;
    if ~(analysissettings.targetchannel == 1 || analysissettings.targetchannel == 2 || analysissettings.targetchannel == 3 || analysissettings.targetchannel == 4)
        error('Invalid target channel value');
    end
else
    analysissettings.targetchannel = 2;
end
% Rotate mixed
if nargin > 1 && isfield(varargin{1},'rotatemixed')
    analysissettings.rotatemixed = varargin{1}.rotatemixed;
    if ~(analysissettings.rotatemixed == 0 || analysissettings.rotatemixed == 1)
        error('Invalid rotate mixed value!');
    end
else
    analysissettings.rotatemixed = 1;
end
% Rotate detail
if nargin > 1 && isfield(varargin{1},'rotatedetail')
    analysissettings.rotatedetail = varargin{1}.rotatedetail;
    if ~(analysissettings.rotatedetail == 0 || analysissettings.rotatedetail == 1)
        error('Invalid rotate detail value!');
    end
else
    analysissettings.rotatedetail = 1;
end
% Lower wavelength cutoff.
if nargin > 1 && isfield(varargin{1},'lowlimit')
    analysissettings.islowlimitauto = 0;
    analysissettings.lowlimit = varargin{1}.lowlimit;
    if isempty(analysissettings.lowlimit) || ~isnumeric(analysissettings.lowlimit) || ~isfinite(analysissettings.lowlimit) || analysissettings.lowlimit <= 0
        error('Invalid lower wavelength limit value!');
    end
    if analysissettings.lowlimit < 2*analysissettigs.imagepixelsize
        analysissettings.lowlimit = 2*analysissettings.imagepixelsize;
        warning('Minimum wavelength is limited by the sampling theorem and has been set equal to resolution doubled!');
    end
else
    analysissettings.islowlimitauto = 1;
    analysissettings.lowlimit = [];
end
% Higher wavelength cutoff.
if nargin > 1 && isfield(varargin{1},'highlimit')
    analysissettings.ishighlimitauto = 0;
    analysissettings.highlimit = varargin{1}.highlimit;
    if isempty(analysissettings.highlimit) || ~isnumeric(analysissettings.highlimit) || ~isfinite(analysissettings.highlimit) || analysissettings.highlimit <= 0
        error('Invalid higher wavelength limit value!');
    end
else
    analysissettings.ishighlimitauto = 1;
    analysissettings.highlimit = [];
end
if nargin > 1 && isfield(varargin{1},'lowlimit') && isfield(varargin{1},'highlimit')
    if analysissettings.lowlimit >= analysissettings.highlimit
        error('Invalid wavelength limits: lower limit equals or exceeds higher limit!');
    end
end
% Exclusion limits.
if nargin > 1 && isfield(varargin{1},'excludelow') && isfield(varargin{1},'excludehigh')
    analysissettings.isexcludeon = 1;
    analysissettings.excludelow = varargin{1}.excludelow;
    analysissettings.excludehigh = varargin{1}.excludehigh;
    if isempty(analysissettings.excludelow) || ~isnumeric(analysissettings.excludelow) || ~isfinite(analysissettings.excludelow) || analysissettings.excludelow <= 0
        error('Invalid lower exclusion limit value!');
    end
    if isempty(analysissettings.excludehigh) || ~isnumeric(analysissettings.excludehigh) || ~isfinite(analysissettings.excludehigh) || analysissettings.excludehigh <= 0
        error('Invalid higher exclusion limit value!');
    end
    if analysissettings.excludelow >= analysissettings.excludehigh
        error('Minimum wavelength to exclude has to be lower than maximum wavelength to exclude!');
    end
    if ~analysissettings.islowlimitauto && analysissettings.excludelow < analysissettings.lowlimit
        error('Wavelengths to exclude should be within the wavelength range!');
    end
    if ~analysissettings.ishighlimitauto && analysissettings.excludehigh > analysissettings.highlimit
        error('Wavelengths to exclude should be within the wavelength range!');
    end
    if analysissettings.excludelow < 2*analysissettigs.imagepixelsize
        analysissettings.excludelow = 2*analysissettings.imagepixelsize;
        warning('Minimum wavelength to exclude is limited by the sampling theorem and has been set equal to resolution doubled!');
    end
else
    analysissettings.isexcludeon = 0;
    analysissettings.excludelow = [];
    analysissettings.excludehigh = [];
end
% Detail component guess.
if nargin > 1 && isfield(varargin{1},'lowguess') && isfield(varargin{1},'highguess')
    analysissettings.isdetailauto = 0;
    analysissettings.lowguess = varargin{1}.lowguess;
    analysissettings.highguess = varargin{1}.highguess;
    if isempty(analysissettings.lowguess) || ~isnumeric(analysissettings.lowguess) || ~isfinite(analysissettings.lowguess) || analysissettings.lowguess <= 0
        error('Invalid detail component wavelengths!');
    end
    if isempty(analysissettings.highguess) || ~isnumeric(analysissettings.highguess) || ~isfinite(analysissettings.highguess) || analysissettings.highguess <= 0
        error('Invalid detail component wavelengths!');
    end
    if analysissettings.lowguess >= analysissettings.highguess
        error('Lower guess for detail component wavelength must be lower than higher guess!');
    end
    if ~analysissettings.islowlimitauto && analysissettings.lowguess < analysissettings.lowlimit
        error('Guesses for detail component wavelength should be within the wavelength range!');
    end
    if ~analysissettings.ishighlimitauto && analysissettings.highguess > analysissettings.highlimit
        error('Guesses for detail component wavelength should be within the wavelength range!');
    end
    if analysissettings.lowguess < 2*analysissettigs.imagepixelsize
        analysissettings.lowguess = 2*analysissettings.imagepixelsize;
        warning('Lower guess for detail component wavelength is limited by the sampling theorem and has been set equal to resolution doubled!');
    end
else
    analysissettings.isdetailauto = 1;
    analysissettings.lowguess = [];
    analysissettings.highguess = [];
end
if ~analysissettings.isdetailauto && analysissettings.isexcludeon && ...
    ((analysissettings.lowguess >= analysissettings.excludelow && analysissettings.lowguess <= analysissettings.excludehigh) || ...
    (analysissettings.highguess >= analysissettings.excludelow && analysissettings.highguess <= analysissettings.excludehigh))
    warning('Guesses for detail component wavelength range overlap with the excluded wavelength range!');
end
% Additional settings.
analysissettings.isdetaildefault = 0;
if nargin > 1 && isfield(varargin{1},'segmentlength')
    analysissettings.segmentlength = varargin{1}.segmentlength;
    if isempty(analysissettings.segmentlength) || ~isnumeric(analysissettings.segmentlength) || ~isfinite(analysissettings.segmentlength) || analysissettings.segmentlength >= 1 || analysissettings.segmentlength <= 0
        error('Invalid segment length value!');
    end
else
    analysissettings.segmentlength = 0.1;
end
if nargin > 1 && isfield(varargin{1},'alpha')
    analysissettings.alpha = varargin{1}.alpha;
    if isempty(analysissettings.alpha) || ~isnumeric(analysissettings.alpha) || ~isfinite(analysissettings.alpha) || analysissettings.alpha >= 1 || analysissettings.alpha <= 0
        error('Invalid significance level value!');
    end
else
    analysissettings.alpha = 0.05;
end
if nargin > 1 && isfield(varargin{1},'convergence_threshold')
    analysissettings.convergence_threshold = varargin{1}.convergence_threshold;
    if isempty(analysissettings.convergence_threshold) || ~isnumeric(analysissettings.convergence_threshold) || ~isfinite(analysissettings.convergence_threshold) || analysissettings.convergence_threshold < 0
        error('Invalid convergence limit value!');
    end
else
    analysissettings.convergence_threshold = 1;
end
if nargin > 1 && isfield(varargin{1},'maxiterations')
    analysissettings.maxiterations = varargin{1}.maxiterations;
    if isempty(analysissettings.maxiterations) || ~isnumeric(analysissettings.maxiterations) || ~isfinite(analysissettings.maxiterations) || analysissettings.maxiterations < 0
        error('Invalid maximum iterations value!');
    end
else
    analysissettings.maxiterations = 10;
end
if nargin > 1 && isfield(varargin{1},'angleslack')
    analysissettings.angleslack = varargin{1}.angleslack;
    if isempty(analysissettings.angleslack) || ~isnumeric(analysissettings.angleslack) || ~isfinite(analysissettings.angleslack) || analysissettings.angleslack >= 90 || analysissettings.segmentlength < 0
        error('Invalid angular tolerance value!');
    end
else
    analysissettings.angleslack = 10;
end
if nargin > 1 && isfield(varargin{1},'sigma_prior_angle')
    analysissettings.sigma_prior_angle = varargin{1}.sigma_prior_angle;
    if isempty(analysissettings.sigma_prior_angle) || ~isnumeric(analysissettings.sigma_prior_angle) || ~isfinite(analysissettings.sigma_prior_angle) || analysissettings.sigma_prior_angle <= 0
        error('Invalid prior width value!');
    end
else
    analysissettings.sigma_prior_angle = 0.5;
end

% Segmentation.
if analysissettings.issegmentationon
    cellregion = varargin{2} > 0;
else
    cellregion = [];
end

% Main analysis.
[result,~,~] = mainAnalysis(inputimage,cellregion,analysissettings);


