function [PSD_polar, frequencies_polar, theta_polar] = PSDtoPolar(PSD_cartesian, frequencies_x, frequencies_y, anglerange, varargin)

% Copyright 2015 Kimmo Kartasalo
% Tampere University of Technology, Tampere, Finland
% http://www.tut.fi/cytospectre
% Email: kimmo.kartasalo@tut.fi
% 
% This file is part of CytoSpectre.
% 
% CytoSpectre is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CytoSpectre is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with CytoSpectre.  If not, see <http://www.gnu.org/licenses/>.

% If number of angle bins is not given, set the number so as to retain
% or exceed the angular bin spacing of the cartesian spectrum.
if nargin < 6
    % Radius of the spectral circle.
    radius = max(length(frequencies_x)/2, length(frequencies_y)/2);
    % Number of points on the circle (2*pi*r corresponds to an angular
    % range of 2*pi).
    N_angles = ceil(anglerange*radius);
else
    N_angles = varargin{2};
end

% If number of frequency bins is not given, set the number so as to
% retain or exceed the frequency bin spacing of the cartesian spectrum.
% Zero frequency bin will be left out. The original number of frequency
% bins should be odd i.e. zero frequency is the middle frequency.
if nargin < 5
    N_frequencies = floor(max(length(frequencies_x)/2, length(frequencies_y)/2));
else
    N_frequencies = varargin{1};
end

% Form vector of angles from 0 to 'anglerange'.
theta_polar = (0:N_angles-1)'/N_angles*anglerange;

% Form vector of radial frequencies. Accept the lower of the maximal
% frequencies along the two dimensions as the maximum frequency. Accept the
% higher of the smallest positive frequencies along the two dimensions as
% the minimum frequency. This will get rid of the zero frequency bin.
maximumfrequency = min(max(frequencies_x), max(frequencies_y));
minimumfrequency = max(min(frequencies_x(frequencies_x > 0)), min(frequencies_y(frequencies_y > 0)));
frequencies_polar = linspace(minimumfrequency,maximumfrequency,N_frequencies)';

% Points on the polar spectrum.
xpolar = cos(theta_polar)*frequencies_polar'; 
ypolar = sin(theta_polar)*frequencies_polar';

% Points on the cartesian spectrum.
[xcart, ycart] = meshgrid(frequencies_x, frequencies_y);
% Transform from cartesian to polar via bilinear interpolation,
% while preserving total variance.
PSD_polar = (1/(N_frequencies*N_angles))*interp2(xcart, ycart, numel(PSD_cartesian)*PSD_cartesian, xpolar, ypolar);

% Perform scaling to counter the effect of different density of spectral
% points at different distances from the origin.
for ind=1:length(frequencies_polar)
    PSD_polar(:,ind) = (pi/2*frequencies_polar(ind)/maximumfrequency)*PSD_polar(:,ind);
end
